//
// 3D Ray-AABB Intersection Maths for GLSL 
// Reference: https://www.reddit.com/r/opengl/comments/8ntzz5/fast_glsl_ray_box_intersection/
// Version: 2020-08-01
//
// Copyright (c) 2020 Ngo Duy Anh. All rights reserved.
// Distributed under the MIT license. See LICENSE file.
// https://github.com/ngoduyanh/glsl-intersect
//

bool rayAABBInv(vec3 rayOrigin, vec3 rayInvDir, vec3 boxMin, vec3 boxMax, out vec2 result) {
    vec3 tbot = rayInvDir * (boxMin - rayOrigin);
    vec3 ttop = rayInvDir * (boxMax - rayOrigin);
    vec3 tmin = min(ttop, tbot);
    vec3 tmax = max(ttop, tbot);
    vec2 t = max(tmin.xx, tmin.yz);
    float t0 = max(t.x, t.y);
    t = min(tmax.xx, tmax.yz);
    float t1 = min(t.x, t.y);
    result = vec2(t0, t1);
    return t1 > max(t0, 0.0);
}

#pragma glslify: export(rayAABBInv)